package com.qima.gift.appcommon.security.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.stp.SaLoginModel;
import cn.dev33.satoken.stp.StpUtil;
import com.qima.gift.appcommon.account.AccountConst;
import com.qima.gift.appcommon.account.bean.AccountAuth;
import com.qima.gift.appcommon.account.bean.AccountInfoBO;
import com.qima.gift.appcommon.account.service.IAccountService;
import com.qima.gift.appcommon.security.SecurityConst;
import com.qima.gift.appcommon.security.satoken.LoginDataDTO;
import com.qima.gift.appcommon.security.service.ISecurityService;
import com.qima.gift.appcommon.security.utils.SecurityUtils;
import com.qima.gift.appcommon.wechat.service.IWeChatService;
import com.qima.gift.common.Result;
import com.qima.gift.common.utils.JsonUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.Builder;
import lombok.Data;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/4 18:42
 */
@RestController
@Tag(name="安全模块")
public class SecurityController {

    @Autowired
    private ISecurityService securityService;

    @Autowired
    private IAccountService accountService;

    @Operation(summary = "WEB认证", description = "内容格式：{\"p\":\"用户名\",\"c\":\"密码\"}")
    @PostMapping("/login")
    public Result<LoginDataDTO> login(@RequestBody String json, HttpServletRequest request) {
        Map<String,String> m = JsonUtils.toObject(json,Map.class);
        String p = m.get("p");
        String c = m.get("c");
        Result<String> r = accountService.doAuth(AccountConst.AccountAuthTypeEnum.UserName, p, c);
        if(r.checkOk()) {
            String accountId = r.getData();
            StpUtil.login(accountId, SecurityConst.DEVICE_PC);
            LoginDataDTO dto = securityService.getLoginData();
            return Result.ok("登录成功", dto);
        } else {
            return Result.fail("登录失败");
        }
    }

    @Operation(summary = "WEB后台认证", description = "内容格式：{\"p\":\"用户名\",\"c\":\"密码\"}")
    @PostMapping("/login_back")
    public Result<LoginDataDTO> loginBack(@RequestBody String json, HttpServletRequest request) {
        Map<String,String> m = JsonUtils.toObject(json,Map.class);
        String p = m.get("p");
        String c = m.get("c");
        if(AccountConst.ACCOUNT_SUPER_USERNAME.equals(p)) {
            Result<String> r = accountService.doAuth(AccountConst.AccountAuthTypeEnum.UserName, p, c);
            if(r.checkOk()) {
                String accountId = r.getData();
                StpUtil.login(accountId, SecurityConst.DEVICE_PC);
                LoginDataDTO dto = securityService.getLoginData();
                return Result.ok("登录成功", dto);
            } else {
                return Result.fail("登录失败");
            }
        } else {
            return Result.fail("登录失败，本入口仅支持管理员登录");
        }
    }

    @Operation(summary = "微信认证", description = "内容格式：{\"code\":\"wx.login获取到的临时凭证\"}")
    @PostMapping("/login_wechat")
    public Result<LoginDataDTO> loginWechat(@RequestBody String json, HttpServletRequest request) {
        Map<String,String> m = JsonUtils.toObject(json,Map.class);
        String code = m.get("code");
        Result<String> r = accountService.doAuth(AccountConst.AccountAuthTypeEnum.WeChat,code, code);
        if(r.checkOk()) {
            String accountId = r.getData();
            StpUtil.login(accountId, SecurityConst.DEVICE_MINI);
            LoginDataDTO dto = securityService.getLoginData();
            return Result.ok("登录成功", dto);
        } else {
            return Result.ok("登录失败", null);
        }
    }

    @GetMapping("/logininfo")
    @Operation( summary = "获取登录信息" )
    @SaCheckLogin
    public Result<LoginDataDTO> loginInfo() {
        LoginDataDTO r = securityService.getLoginData();
        return Result.ok("登录信息获取成功",r);
    }

    @GetMapping("/logout")
    @Operation( summary = "登出" )
    public Result<String> logout() {
        StpUtil.logout();
        return Result.ok("登出成功",null);
    }

}
