package com.qima.gift.appcommon.space.base;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import com.google.common.io.BaseEncoding;
import com.google.common.io.ByteSource;
import com.google.common.io.CharSource;
import com.qima.gift.common.AppException;
import com.qima.gift.common.CommonConst;
import org.apache.commons.lang3.StringUtils;

import java.io.*;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/13 10:37
 */
public class Base64FileSource implements IFileSource{

    private String fileName;
    private String contentType;
    /**
     * base64编码的数据
     */
    private String content;

    private ByteSource byteSource;

    @Override
    public String getFileName() {
        return fileName;
    }

    @Override
    public String getContentType() {
        findByteSource();
        return contentType;
    }

    public ByteSource findByteSource() {
        if(StringUtils.isBlank(content)) { return null;}
        if(byteSource == null) {
            String trueContent = content;
            String subContent = content.length()>256?content.substring(0,256):content;
            int i = subContent.indexOf(CommonConst.COMMA_STRING);
            if(i > 0) {
                String headerStr = subContent.substring(0,i);
                int p1 = headerStr.indexOf(":");
                int p2 = headerStr.indexOf(";");
                contentType = headerStr.substring(p1+1, p2);
                String base = headerStr.substring(p2+1, i);
                trueContent = content.substring(i+1);
                if(!"base64".equals(base)){
                    throw new AppException("不能处理非base64的数据").setM(base);
                }
            }
            CharSource charSource = CharSource.wrap(trueContent);
            byteSource = BaseEncoding.base64().decodingSource(charSource);
        }
        return byteSource;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    @Override
    public InputStream getInputStream() throws IOException {
        if(StringUtils.isBlank(content)) { return null;}
        ByteSource bytes = findByteSource();
        return bytes.openStream();
    }
}
