package com.qima.gift.appcommon.space.service;

import com.google.common.collect.Lists;
import com.qima.gift.appcommon.space.base.IFileSource;
import com.qima.gift.appcommon.space.bean.SpaceFile;
import com.qima.gift.appcommon.space.bean.SpaceFileDTO;
import com.qima.gift.common.AppException;
import com.qima.gift.common.utils.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.io.InputStreamSource;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/9 11:26
 */
public interface ISpaceService {

    /**
     * 二段提交之临时文件创建
     * @param source 资源
     * @return
     */
    default SpaceFile createSpaceFileOfTmp(IFileSource source) {
        if(source == null) {throw new AppException("源资源不能为空");}
        List<SpaceFile> fs = createSpaceFileOfTmp(Lists.newArrayList(source));
        return CollectionUtils.isEmpty(fs)?null:fs.stream().findFirst().get();
    }

    /**
     * 二段提交之临时文件创建
     * @param sources
     * @return
     */
    List<SpaceFile> createSpaceFileOfTmp(List<? extends IFileSource> sources);

    /**
     * 正式创建
     * @param source
     * @return
     */
    default SpaceFile createSpaceFile(IFileSource source) {
        if(source == null) {throw new AppException("源资源不能为空");}
        List<SpaceFile> fs = createSpaceFile(Lists.newArrayList(source));
        return CollectionUtils.isEmpty(fs)?null:fs.stream().findFirst().get();
    }

    /**
     * 正式创建
     * @param source
     * @return
     */
    List<SpaceFile> createSpaceFile(List<? extends IFileSource> source);

    /**
     * 二段提交之正式提交
     * @param fileId
     * @return
     */
    default SpaceFile doFormal(String fileId) {
        if(StringUtils.isBlank(fileId)) {throw new AppException("文件不能为空");}
        List<SpaceFile> fs = doFormal(Lists.newArrayList(fileId));
        return CollectionUtils.isEmpty(fs)?null:fs.stream().findFirst().get();
    }

    /**
     * 二段提交之正式提交
     * @param fileIds
     * @return
     */
    List<SpaceFile> doFormal(List<String> fileIds);

    Boolean deleteSpaceFile(String fileId);

    /**
     * 查找文件信息
     * @param fileId
     * @return
     */
    SpaceFile findSpaceFile(String fileId);

    /**
     * 读取文件
     * @param fileId
     * @return
     */
    IFileSource findSpaceFileAsStream(String fileId);


}
