package com.qima.gift.appcommon.system.service;

import com.qima.gift.appcommon.system.SystemConst;
import com.qima.gift.appcommon.system.bean.AppSystemProperties;
import com.qima.gift.appcommon.system.bean.SystemInfo;
import com.qima.gift.appcommon.system.bean.SystemInfoExample;
import com.qima.gift.appcommon.system.dao.SystemInfoMapper;
import com.qima.gift.appcommon.system.utils.SystemUtils;
import com.qima.gift.common.AppException;
import com.qima.gift.common.utils.CollectionUtils;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/19 10:10
 */
@Component
@Slf4j
public class SystemService {

    @Autowired
    private AppSystemProperties appSystemProperties;
    @Autowired
    private SystemInfoMapper systemInfoMapper;

    @PostConstruct
    public void init(){
        SystemInfo dbPlat = systemInfoMapper.selectByPrimaryKey(SystemConst.SYSTEM_DEFAULT_ID);
        if(dbPlat == null) {
            SystemInfo plat = new SystemInfo();
            plat.setName(appSystemProperties.getName());
            plat.setOtherName(appSystemProperties.getCompany());
            plat.setSystemType(SystemConst.SystemInfoTypeEnum.Plat.getKey());
            createSystemInfo(plat);
        } else {
            dbPlat.setName(appSystemProperties.getName());
            dbPlat.setOtherName(appSystemProperties.getCompany());
            updateSystemInfo(dbPlat);
        }
    }

    public SystemInfo createSystemInfo(SystemInfo sys) {
        if(sys == null){
            throw new AppException("系统不能为空");
        }
        if(sys.getName() == null){
            throw new AppException("系统名不能为空");
        }
        SystemInfoExample e = new SystemInfoExample();
        e.createCriteria().andNameEqualTo(sys.getName());
        List<SystemInfo> dbNameInfos = systemInfoMapper.selectByExample(e);
        SystemInfo dbNameInfo = CollectionUtils.isNotEmpty(dbNameInfos)?dbNameInfos.stream().findFirst().get():null;
        if(dbNameInfo != null) {
            throw new AppException("系统不允许重名").setM(sys.getName());
        }
        SystemUtils.fillBeanBeforeInsert(sys);
        systemInfoMapper.insert(sys);
        log.info("系统["+sys.getName()+"]创建完成");
        return sys;
    }

    public SystemInfo updateSystemInfo(SystemInfo sys) {
        if(sys == null){
            throw new AppException("系统不能为空");
        }
        if(sys.getName() == null){
            throw new AppException("系统名不能为空");
        }
        SystemInfo dbInfo = systemInfoMapper.selectByPrimaryKey(sys.getId());
        if(dbInfo == null){
            throw new AppException("系统不存在").setM(sys.getId());
        }
        SystemInfoExample e = new SystemInfoExample();
        e.createCriteria().andNameEqualTo(sys.getName());
        List<SystemInfo> dbNameInfos = systemInfoMapper.selectByExample(e);
        SystemInfo dbNameInfo = CollectionUtils.isNotEmpty(dbNameInfos)?dbNameInfos.stream().findFirst().get():null;
        if(dbNameInfo != null && !dbNameInfo.getId().equals(dbInfo.getId())) {
            throw new AppException("系统不允许重名").setM(sys.getName());
        }
        SystemUtils.fillBeanBeforeUpdate(dbInfo);
        dbInfo.setName(sys.getName());
        dbInfo.setOtherName(sys.getOtherName());
        systemInfoMapper.updateByPrimaryKey(dbInfo);
        log.info("系统["+sys.getName()+"]更新完成");
        return dbInfo;
    }

}
