package com.qima.gift.appcommon.wechat.bean;

import com.qima.gift.common.utils.JsonUtils;

import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/4 12:30
 */
public abstract class AbsWeChatBean<Q,S> {

    public static final String METHOD_GET = "get";
    public static final String METHOD_POST = "post";

    protected String name;

    protected String url;

    protected String method;

    protected Q request;

    protected S response;

    protected int retry = 0;

    protected AbsWeChatBean(String n, String u, String m, Q r) {
        this.name = n;
        this.url = u;
        this.method = m;
        this.request = r;
    }

    public int findAndAddRetry(){
        int r = retry;
        retry++;
        return r;
    }

    public int getRetry() {
        return retry;
    }

    public String getName(){
        return name;
    }

    public String getUrl() {
        return url;
    }

    public String getMethod(){
        return method;
    }

    public Q getReqParam() {
        return request;
    }

    public S getResParam() {
        return response;
    }

    public Integer getResParamErrorCode() {
        return response == null?null:(Integer)JsonUtils.toMap(response).get("errcode");
    }

    public void setResParam(S res){
        response = res;
    }

    public Map<String,?> getReqParamOfUrl() {
        return JsonUtils.toMap(getReqParam());
    }

    public Map<String,?> getReqParamOfBody() {
        Map<String,?> r = JsonUtils.toMap(getReqParam());
        if(this instanceof IWeChatNeedToken) {
            r.remove(IWeChatNeedToken.TOKEN_KEY);
        }
        return r;
    }

    /**
     * 获取结果类
     * @return
     */
    public abstract Class<S> getResParamClass();

    /**
     * 判断请求是否OK
     * @return
     */
    public abstract boolean isReqOk();

    /**
     * 判断结果是否OK
     * @return
     */
    public abstract boolean isResOk();

}
