package com.qima.gift.appcommon.wechat.bean;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.qima.gift.common.CommonConst;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/4 12:20
 */
public class GetAccessToken extends AbsWeChatBean<GetAccessToken.ReqParam, GetAccessToken.ResParam> implements IWeChatNeedAppId{

    public GetAccessToken() {
        super("getAccessToken", "https://api.weixin.qq.com/cgi-bin/token?grant_type={grant_type}&appid={appid}&secret={secret}", "GET", new ReqParam());
    }

    @Override
    public Class<ResParam> getResParamClass() {
        return ResParam.class;
    }

    @Override
    public boolean isReqOk() {
        return StringUtils.isNotBlank(request.getGrant_type());
    }

    @Override
    public boolean isResOk() {
        return response != null && (response.errcode == null || Integer.valueOf(CommonConst.INT_0).equals(response.errcode));
    }

    @Override
    public void setAppId(String appid) {
        getReqParam().setAppid(appid);
    }

    @Override
    public void setAppSecret(String appsecret) {
        getReqParam().setSecret(appsecret);
    }

    @Data
    @JsonPropertyOrder({"grant_type", "appid", "secret"})
    public static class ReqParam{
        private String grant_type = "client_credential";  // 填写 client_credential
        private String appid;  // 小程序唯一凭证，即 AppID，可在「微信公众平台 - 设置 - 开发设置」页中获得。（需要已经成为开发者，且帐号没有异常状态）
        private String secret;  // 小程序唯一凭证密钥，即 AppSecret，获取方式同 appid
    }

    @Data
    public static class ResParam{
        private Integer errcode;// 错误时：错误码
        private String errmsg;// 错误时：错误信息
        private String access_token;  // 正确时：获取到的凭证
        private int expires_in;  // 正确时：凭证有效时间，单位：秒。目前是7200秒之内的值
    }
}
