package com.qima.gift.appcommon.wechat.bean;

import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.qima.gift.common.CommonConst;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/4 12:50
 */
public class GetPhoneNumber extends AbsWeChatBean<GetPhoneNumber.ReqParam, GetPhoneNumber.ResParam> implements IWeChatNeedToken {

    public GetPhoneNumber() {
        super("getPhoneNumber", "https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token={access_token}", METHOD_POST, new GetPhoneNumber.ReqParam());
    }

    @Override
    public void setToken(String token) {
        getReqParam().setAccess_token(token);
    }

    @Override
    public Class<ResParam> getResParamClass() {
        return ResParam.class;
    }

    @Override
    public boolean isReqOk() {
        return StringUtils.isNotBlank(request.getCode());
    }

    @Override
    public boolean isResOk() {
        return response != null && (response.errcode == null || Integer.valueOf(CommonConst.INT_0).equals(response.errcode));
    }

    @Data
    @JsonPropertyOrder({"access_token", "code"})
    public static class ReqParam{
        private String access_token;  // 接口调用凭证，该参数为 URL 参数，非 Body 参数。使用access_token或者authorizer_access_token
        private String code;  // 手机号获取凭证
    }

    @Data
    public static class ResParam{
        private Integer errcode;  // 错误码
        private String errmsg;  // 错误信息
        private PhoneInfo phone_info;  // 用户手机号信息
    }

    @Data
    public static class PhoneInfo {
        private String phoneNumber;  // 用户绑定的手机号（国外手机号会有区号）
        private String purePhoneNumber;  // 没有区号的手机号
        private String countryCode;  // 区号
        private Watermark watermark;  // 数据水印
    }

    @Data
    public static class Watermark {
        private Long timestamp;  // 用户获取手机号操作的时间戳
        private String appid;  // 小程序appid
    }

}
