package com.qima.gift.appcommon.wechat.service;

import com.qima.gift.appcommon.account.service.IAccountService;
import com.qima.gift.appcommon.wechat.WeChatEnum;
import com.qima.gift.appcommon.wechat.authc.WeChatAccountAuthc;
import com.qima.gift.appcommon.wechat.bean.*;
import com.qima.gift.common.AppException;
import com.qima.gift.common.CommonConst;
import com.qima.gift.common.FixedLinkedList;
import com.qima.gift.common.utils.HttpUtils;
import com.qima.gift.common.utils.JsonUtils;
import io.swagger.v3.core.util.Json;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.C;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/3 20:16
 */
@Service
@Slf4j
public class WeChatServiceImpl implements IWeChatService{

    @Autowired
    private IAccountService accountService;

    @Autowired
    private WeChatConfigBean weChatConfigBean;

    private FixedLinkedList<String> accessTokens = new FixedLinkedList<String>(CommonConst.INT_5);

    @Override
    public WeChatConfigBean findConfig() {
        return weChatConfigBean;
    }

    @Override
    public String findAccessToken() throws AppException {
        if(accessTokens.isEmpty()) {
            refreshAccessToken();
        }
        if(accessTokens.isEmpty()) {
            throw new AppException("微信accessToken获取失败");
        } else {
            return accessTokens.getLast();
        }
    }

    @Override
    public String code2Session(String jsCode) throws AppException {
        Code2Session c = new Code2Session();
        c.getReqParam().setJs_code(jsCode);
        callWeChat(c);
        return c.getResParam().getOpenid();
    }

    @Override
    public String getPhoneNumber(String code) throws AppException {
        GetPhoneNumber p = new GetPhoneNumber();
        p.getReqParam().setCode(code);
        callWeChat(p);
        return p.getResParam().getPhone_info().getPhoneNumber();
    }

    @Override
    public <Q,S> void callWeChat(AbsWeChatBean<Q,S> i) throws AppException {
        if(i == null) {
            throw new AppException("微信请求不能为空");
        } else if (!i.isReqOk()) {
            throw new AppException("微信请求构建错误");
        } else {
            if (i instanceof IWeChatNeedAppId) {
                ((IWeChatNeedAppId) i).setAppId(findConfig().getAppId());
                ((IWeChatNeedAppId) i).setAppSecret(findConfig().getAppSecret());
            }
            if (i instanceof IWeChatNeedToken) {
                ((IWeChatNeedToken) i).setToken(findAccessToken());
            }
            log.info("微信接口["+i.getName()+"]开始发送，json="+JsonUtils.toJson(i));
            if (AbsWeChatBean.METHOD_GET.equals(i.getMethod())) {
                S o = HttpUtils.getSend(i.getUrl(),i.getResParamClass(),i.getReqParamOfUrl());
                i.setResParam(o);
            } else {
                S o = HttpUtils.postSend(i.getUrl(),i.getResParamClass(),i.getReqParamOfUrl(),i.getReqParamOfBody());
                i.setResParam(o);
            }
            if(i.isResOk()){
                log.info("微信接口["+i.getName()+"]执行成功，json="+JsonUtils.toJson(i));
            } else {
                log.warn("微信接口["+i.getName()+"]发生错误，json="+JsonUtils.toJson(i));
                if(WeChatEnum.ResultCode.isTokenError(i.getResParamErrorCode())
                        && !GetAccessToken.class.equals(i.getClass())
                        && !GetStableAccessToken.class.equals(i.getClass())
                        &&i.findAndAddRetry() < CommonConst.INT_2){
                    log.warn("微信接口["+i.getName()+"]重新尝试["+i.getRetry()+"]，json="+JsonUtils.toJson(i));
                    refreshAccessToken();
                    callWeChat(i);
                }
            }
        }
    }

//    @Scheduled(cron = "0 0/59 * * * ?")
//    public void refreshAccessToken() {
//        log.info("请求微信accessToken开始");
//        GetAccessToken token = new GetAccessToken();
//        callWeChat(token);
//        if(token.isResOk()) {
//            String t = token.getResParam().getAccess_token();
//            accessTokens.add(t);
//            log.info("请求微信accessToken成功，res=："+JsonUtils.toJson(token.getResParam())+"；当前列表：" + JsonUtils.toJson(accessTokens));
//        } else {
//            log.warn("请求微信accessToken失败，res="+JsonUtils.toJson(token.getResParam()));
//        }
//    }

    @Scheduled(cron = "0 0/5 * * * ?")
    public void refreshAccessToken() {
        log.info("请求微信accessToken开始");
        GetStableAccessToken token = new GetStableAccessToken();
        callWeChat(token);
        if(token.isResOk()) {
            String t = token.getResParam().getAccess_token();
            accessTokens.add(t);
            log.info("请求微信accessToken成功，res=："+JsonUtils.toJson(token.getResParam())+"；当前列表：" + JsonUtils.toJson(accessTokens));
        } else {
            log.warn("请求微信accessToken失败，res="+JsonUtils.toJson(token.getResParam()));
        }
    }
}
