package com.qima.gift.common.utils;

import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.qima.gift.common.AppException;
import com.qima.gift.common.CommonConst;
import com.qima.gift.common.page.Page;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.function.Function;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 * 分页的工具类
 * @author King
 * @since 2023/6/8 17:47
 */
public class PageUtils {

    public static <P> Page<P> buildParam(Object pageNumber, Object pageSize) {
        Page<P> r = new Page<P>();
        if(ObjectUtils.allNotNull(pageNumber)) {
            r.setPageNum(Integer.valueOf(pageNumber.toString()));
        }
        if(ObjectUtils.allNotNull(pageSize)) {
            r.setPageSize(Integer.valueOf(pageSize.toString()));
        }
        return r;
    }

    public static <P> Page<P> buildResult(Page param, Object total, List<P> datas) {
        Page<P> r = new Page<P>();
        r.setPageNum(param.getPageNum());
        r.setPageSize(param.getPageSize());
        if(ObjectUtils.allNotNull(total)) {
            r.setTotal(Integer.valueOf(total.toString()));
        }
        if(ObjectUtils.allNotNull(datas)) {
            r.setDatas(Lists.newArrayList(datas));
        }
        return r;
    }

    /**
     * 拷贝Page
     * @param page
     * @param converter
     * @return
     */
    public static <P,R> Page<R> convertPage(Page<P> page, Function<P, R> converter) {
        Page<R> r = new Page<R>();
        r.setPageSize(page.getPageSize());
        r.setPageNum(page.getPageNum());
        r.setTotal(page.getTotal());
        List<R> rs = Lists.newArrayList();
        if(!CollectionUtils.isEmpty(page.getDatas()) && converter != null) {
            page.getDatas().forEach(e->rs.add(converter.apply(e)));
        }
        return r;
    }

    public static <P,R> PageInfo<R> convertPage(PageInfo<P> page, Function<P, R> converter) {
        PageInfo<R> r = new PageInfo<R>();
        r.setPageSize(page.getPageSize());
        r.setPageNum(page.getPageNum());
        r.setTotal(page.getTotal());
        List<R> rs = Lists.newArrayList();
        if(!CollectionUtils.isEmpty(page.getList()) && converter != null) {
            page.getList().forEach(e->rs.add(converter.apply(e)));
        }
        r.setList(rs);
        return r;
    }

    /**
     * 根据前台传递的数据按照pg的标准生成LIMIT的数据
     * @param pageSize
     * @param page
     * @return
     */
    public static Integer[] convertPGLimit(Integer pageSize,Integer page) {
        pageSize = pageSize == null? CommonConst.INT_20:pageSize;
        page = page == null?CommonConst.INT_1:page;
        Integer start = (page-CommonConst.INT_1) * pageSize;
        return new Integer[] {pageSize,start};
    }

    /**
     * 标准话参数
     * @param pageSize
     * @param page
     * @return
     */
    public static Integer[] convertStandardPage(Object pageSize,Object page) {
        Integer pageSize1 = pageSize == null?CommonConst.INT_20:Integer.valueOf(pageSize.toString());
        Integer page1 = page == null?CommonConst.INT_1:Integer.valueOf(page.toString());
        return new Integer[] {pageSize1,page1};
    }

    public static Integer[] convertMYSQLLimit(Integer pageSize,Integer page, Integer count) {
        pageSize = pageSize == null?CommonConst.INT_20:pageSize;
        page = page == null?CommonConst.INT_1:page;
        Integer start = (page-CommonConst.INT_1) * pageSize;
        Integer end = start + pageSize;
        if(count != null) {
            start = start > count?count:start;
            end = end > count?count:end;
            if(start > end) {
                throw new AppException("分页参数不对");
            }
        }
        return new Integer[] {start,end};
    }
}
