package com.qima.gift.mall.security;

import cn.dev33.satoken.listener.SaTokenListenerForSimple;
import cn.dev33.satoken.stp.SaLoginModel;
import cn.dev33.satoken.stp.StpUtil;
import com.qima.gift.appcommon.security.base.ISecurityListener;
import com.qima.gift.appcommon.security.satoken.LoginDataDTO;
import com.qima.gift.appcommon.security.service.ISecurityService;
import com.qima.gift.appcommon.security.utils.SecurityUtils;
import com.qima.gift.common.AppException;
import com.qima.gift.common.CommonConst;
import com.qima.gift.common.utils.CollectionUtils;
import com.qima.gift.common.utils.ObjectUtils;
import com.qima.gift.mall.MallConst;
import com.qima.gift.mall.bean.MallMerchant;
import com.qima.gift.mall.bean.MallMerchantMember;
import com.qima.gift.mall.service.IMallService;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/11 20:44
 */
@Component
@Slf4j
public class MallSecurityListener implements ISecurityListener {

    @Autowired
    private IMallService mallService;

    @Autowired
    private ISecurityService securityService;

    @PostConstruct
    public void init() {
        securityService.registSecurityListener(this);
    }

    @Override
    public void onLogin(LoginDataDTO d) {
        onLoad(d);
    }

    @Override
    public void onLoad(LoginDataDTO d) {
        // 目前仅支持一个员工加入一个商户
        List<MallMerchantMember> members = mallService.findMembersByAccountId(d.getAccountId());
        if(CollectionUtils.isNotEmpty(members)) {
            if(members.size() > CommonConst.INT_1) {
                throw new AppException("目前仅支持一个员工加入一个商户");
            } else {
                MallMerchantMember member = members.stream().findFirst().get();
                MallMerchant merchant = mallService.findMerchantById(member.getMerchantId());
                d.setPhone(member.getPhone());
                d.setAttr(MallConst.SECURITY_ATTR_MERCHANTID, member.getMerchantId());
                d.setAttr(MallConst.SECURITY_ATTR_MERCHANTNAME, merchant.getName());
                d.setAttr(MallConst.SECURITY_ATTR_MEMBERID, member.getId());
                d.setAttr(MallConst.SECURITY_ATTR_MERCHANTSTATE,merchant.getState());
            }
        }
    }
}
