package com.qima.gift.mall.service;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.qima.gift.appcommon.account.AccountConst;
import com.qima.gift.appcommon.account.bean.AccountAuth;
import com.qima.gift.appcommon.account.bean.AccountInfoBO;
import com.qima.gift.appcommon.account.service.IAccountService;
import com.qima.gift.appcommon.space.bean.SpaceFile;
import com.qima.gift.appcommon.space.service.ISpaceService;
import com.qima.gift.common.AppException;
import com.qima.gift.common.CommonConst;
import com.qima.gift.common.utils.CollectionUtils;
import com.qima.gift.common.utils.JsonUtils;
import com.qima.gift.mall.bean.*;
import com.qima.gift.mall.dao.*;
import com.qima.gift.mall.security.MallSecurityListener;
import com.qima.gift.mall.utils.MallUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * Copyright: Copyright (C) 2023 KingTom, Inc. All rights reserved. <p>
 * Company: 成都诚唐科技有限公司<p>
 *
 * @author King
 * @since 2023/6/7 20:50
 */
@Service
@Slf4j
public class MallServiceImpl implements IMallService {

    @Autowired
    private MallShowMapper mallShowMapper;
    @Autowired
    private MallMerchantMapper mallMerchantMapper;
    @Autowired
    private MallMerchantMemberMapper mallMerchantMemberMapper;
    @Autowired
    private MallGoodsMapper mallGoodsMapper;
    @Autowired
    private MallGoodsSkuMapper mallGoodsSkuMapper;
    @Autowired
    private MallCategoryMapper mallCategoryMapper;
    @Autowired
    private ISpaceService spaceService;
    @Autowired
    private IAccountService accountService;

    @Override
    public MallMerchantBO saveMerchant(String company, String name, String phone, String accountId) {
        if(StringUtils.isAnyBlank(company,name,phone)) {
            throw new AppException("公司名、姓名、电话不能为空").setM(JsonUtils.toJson(Lists.newArrayList(company,name,phone)));
        }
        // 这里用商户名+联系人作为商户的唯一区分
        MallMerchant m = findMallMerchantByName(company);
        if(m == null) {
            // 公司还不存在
            MallMerchant newm = new MallMerchant();
            MallUtils.fillBeanBeforeInsert(newm);
            newm.setName(company);
            newm.setExt1(name);
            mallMerchantMapper.insert(newm);
            m = newm;
            log.info("商户["+m+"]不存在，创建了一个新的["+m.getId()+"]");
        } else {
            log.info("商户["+m+"]已存在，直接使用["+m.getId()+"]");
        }
        createMallMerchantMember(m.getId(),name,phone,accountId);
        MallMerchant rbean = mallMerchantMapper.selectByPrimaryKey(m.getId());
        return fillMallMerchantBO(rbean);
    }

    @Override
    public MallMerchantBO updateMerchant(String merchantId, String company, String name, String phone) {
        if(StringUtils.isAnyBlank(company,name,phone)) {
            throw new AppException("商户名、姓名、电话不能为空").setM(JsonUtils.toJson(Lists.newArrayList(company,name,phone)));
        }
        MallMerchant bean = findMerchantById(merchantId);
        if(bean == null) {
            throw new AppException("商户不存在");
        }
        // 这里用商户名+联系人作为商户的唯一区分
        MallMerchant m = findMallMerchantByName(company);
        if(m != null && !m.getId().equals(bean.getId())) {
            throw new AppException("修改的商户已经存在，不允许合并");
        } else {
            // 商户还不存在
            bean.setName(company);
            bean.setExt1(name);
            MallUtils.fillBeanBeforeUpdate(null, bean);
            mallMerchantMapper.updateByPrimaryKey(bean);
            log.info("商户["+m+"]已修改"+JsonUtils.toJson(bean));
        }
        updateMallMerchantMember(bean.getOwner(),name,phone);
        return fillMallMerchantBO(bean);
    }

    @Override
    public MallMerchantBO updateMerchantState(String merchantId, CommonConst.StateEnum state) {
        MallMerchant bean = mallMerchantMapper.selectByPrimaryKey(merchantId);
        if(bean == null) {
            throw new AppException("商户不存在").setM(merchantId);
        } else {
            MallUtils.fillBeanBeforeUpdate(null,bean);
            bean.setState(state.getKey());
            mallMerchantMapper.updateByPrimaryKey(bean);
        }
        return fillMallMerchantBO(bean);
    }

    @Override
    public MallMerchant findMerchantById(String id) {
        if(StringUtils.isAnyBlank(id)) {return null;}
        return mallMerchantMapper.selectByPrimaryKey(id);
    }

    @Override
    public PageInfo<MallMerchantBO> findMerchants(String name, Integer pageNum, Integer pageSize) {
        MallMerchantExample e = new MallMerchantExample();
        if(StringUtils.isNotBlank(name)) {
            e.createCriteria().andNameLike("%"+name+"%");
        }
        PageHelper.startPage(pageNum, pageSize, "gmt_create desc");
        List<MallMerchant> merchants = mallMerchantMapper.selectByExample(e);
        List<MallMerchantBO> merchantBOS = fillMallMerchantBOs(merchants);
        return new PageInfo<>(merchantBOS);
    }

    @Override
    public MallMerchantMember findMemberByAccountId(String merchantId, String accountId) {
        if(StringUtils.isAnyBlank(merchantId,accountId)) {return null;}
        MallMerchantMemberExample e = new MallMerchantMemberExample();
        e.createCriteria().andAccountIdEqualTo(accountId).andMerchantIdEqualTo(merchantId);
        List<MallMerchantMember> ms = mallMerchantMemberMapper.selectByExample(e);
        return CollectionUtils.isEmpty(ms)?null:ms.stream().findFirst().get();
    }

    @Override
    public List<MallMerchantMember> findMembersByAccountId(String accountId) {
        if(StringUtils.isAnyBlank(accountId)) {return null;}
        MallMerchantMemberExample e = new MallMerchantMemberExample();
        e.createCriteria().andAccountIdEqualTo(accountId);
        return mallMerchantMemberMapper.selectByExample(e);
    }

    @Override
    public MallMerchantMember findMemberById(String id) {
        if(StringUtils.isBlank(id)) {return null;}
        return mallMerchantMemberMapper.selectByPrimaryKey(id);
    }

    @Override
    public PageInfo<MallShowBO> findShowsByMerchantId(String merchantId, MallShowQuery query, Integer pageNum, Integer pageSize) {
        MallShowExample e = new MallShowExample();
        MallShowExample.Criteria c = e.createCriteria();
        c.andMerchantIdEqualTo(merchantId);
        if(ObjectUtils.allNotNull(query) && StringUtils.isNotBlank(query.getName())) {
            c.andNameLike("%"+query.getName()+"%");
        }
        e.setOrderByClause("gmt_create");
        PageHelper.startPage(pageNum, pageSize, "gmt_create desc");
        List<MallShow> beans = mallShowMapper.selectByExample(e);
        List<MallShowBO> bos = fillMallShowBOs(beans);
        return new PageInfo<>(bos);
    }

    @Override
    public MallShowBO findShowById(String showId) {
        MallShowBO r = null;
        if(StringUtils.isNotBlank(showId)) {
            MallShow s = mallShowMapper.selectByPrimaryKey(showId);
            r = ObjectUtils.allNotNull(s) ? fillMallShowBO(s):null;
        }
        return r;
    }

    @Override
    public MallShowBO doCopyShow(String showId) {
        MallShowBO b = findShowById(showId);
        if(ObjectUtils.allNotNull(b)) {
            MallShowBO bo = com.qima.gift.common.utils.ObjectUtils.copyObject(b);
            bo.getMallShow().setName(bo.getMallShow().getName()+"-新");
            return createMallShowBO(bo);
        }
        return null;
    }

    @Override
    public MallShowBO createShow(String merchantId, String ownerId, String name, String des) {
        MallMerchant merchant = mallMerchantMapper.selectByPrimaryKey(merchantId);
        MallMerchantMember member = mallMerchantMemberMapper.selectByPrimaryKey(ownerId);
        if(ObjectUtils.allNull(merchant)) {
            throw new AppException("找不到商户").setM(merchantId);
        }
        if(ObjectUtils.allNull(member)) {
            throw new AppException("找不到员工").setM(ownerId);
        }
        MallShow bean = new MallShow();
        bean.setName(name);
        bean.setDes(des);
        bean.setCreater(ownerId);
        bean.setMerchantId(merchantId);
        MallShowBO bo = new MallShowBO(bean);
        return createMallShowBO(bo);
    }

    @Override
    public String doSetShowTopic(String showId, String topicColor, String topicImage, String name, String des) {
        MallShow bean = mallShowMapper.selectByPrimaryKey(showId);
        if( ObjectUtils.allNotNull(bean) ) {
            if(StringUtils.isNotBlank(topicColor)) {
                bean.setTopicColor(topicColor);
            }
            if(StringUtils.isNotBlank(topicImage)) {
                SpaceFile f = spaceService.doFormal(topicImage);
                bean.setTopicImage(f.getId());
            }
            if(StringUtils.isNotBlank(name)) {
                bean.setName(name);
            }
            if(StringUtils.isNotBlank(des)) {
                bean.setDes(des);
            }
            MallUtils.fillBeanBeforeUpdate(null, bean);
            mallShowMapper.updateByPrimaryKey(bean);
            return bean.getId();
        } else {
            throw new AppException("秀不存在").setM(showId);
        }
    }

    @Override
    public String doSetShowContact(String showId, String showContact, String showPhone) {
        MallShow bean = mallShowMapper.selectByPrimaryKey(showId);
        if( ObjectUtils.allNotNull(bean) ) {
            Map<String,String> map = Maps.newHashMap();
            map.put("showContact", showContact);
            map.put("showPhone", showPhone);
            bean.setExt2(JsonUtils.toJson(map));
            MallUtils.fillBeanBeforeUpdate(null, bean);
            mallShowMapper.updateByPrimaryKey(bean);
            return bean.getId();
        } else {
            throw new AppException("秀不存在").setM(showId);
        }
    }

    @Override
    public MallShowBO doSaveShowCategorys(String showId, List<MallCategory> categorys) {
        MallShow bean = mallShowMapper.selectByPrimaryKey(showId);
        if(com.qima.gift.common.utils.ObjectUtils.isNotNull(bean)) {
            if(com.qima.gift.common.utils.ObjectUtils.isNotNull(categorys)) {
                // 先保存分类
                Map<String,String> savedMap = Maps.newLinkedHashMap();
                for(MallCategory category : categorys) {
                    category.setMerchantId(bean.getMerchantId());
                    // 如果有有老ID，就暂时记录，用作关联改变用
                    String oldId = category.getId();
                    MallCategory savedCategory = saveMallCategory(category);
                    savedMap.put(savedCategory.getId(),oldId);
                }
                // 再调整关联
                bean.getContent().getE().resetCategories(savedMap);
                // 最后保存
                MallUtils.fillBeanBeforeUpdate(null,bean);
                mallShowMapper.updateByPrimaryKey(bean);
            }
            return fillMallShowBO(bean);
        } else {
            throw new AppException("秀不存在").setM(showId);
        }
    }

    @Override
    public MallGoodsSku doAddShowGoods(String showId, String categoryId, MallGoodsSku sku) {
        MallShow b = mallShowMapper.selectByPrimaryKey(showId);
        if(b == null) {
            throw new AppException("秀不存在").setM(showId);
        }
        if(!b.getContent().getE().findCategoryExist(categoryId)) {
            throw new AppException("该分类不存在").setM(categoryId);
        }
        sku.setMerchantId(b.getMerchantId());
        MallGoodsSku savedSku = createMallGoodsSku(sku);
        MallShowGoods g = new MallShowGoods();
        g.setCategoryId(categoryId);
        g.setGoodsId(savedSku.getGoodsId());
        g.setSkuId(savedSku.getId());
        b.getContent().getE().getGoods().add(g);
        MallUtils.fillBeanBeforeUpdate(null,b);
        mallShowMapper.updateByPrimaryKey(b);
        return savedSku;
    }

    @Override
    public MallGoodsSku doUpdateShowGoods(String showId, String showGoodsId, MallGoodsSku sku) {
        MallShow b = mallShowMapper.selectByPrimaryKey(showId);
        MallGoodsSku oldSku = mallGoodsSkuMapper.selectByPrimaryKey(showGoodsId);
        if(b == null) {
            throw new AppException("秀不存在").setM(showId);
        }
        if(oldSku == null){
            throw new AppException("商品不存在").setM(showGoodsId);
        }
        sku.setId(showGoodsId);
        sku.setMerchantId(b.getMerchantId());
        return updateMallGoodsSku(sku);
    }

    @Override
    public Boolean doDelShowGoods(String showId, String showGoodsId) {
        MallShow b = mallShowMapper.selectByPrimaryKey(showId);
        if(ObjectUtils.allNotNull(b)) {
            b.getContent().getE().delGoodsSku(showGoodsId);
            mallGoodsSkuMapper.deleteByPrimaryKey(showGoodsId);
            MallUtils.fillBeanBeforeUpdate(null,b);
            mallShowMapper.updateByPrimaryKey(b);
            return Boolean.TRUE;
        } else {
            throw new AppException("秀不存在").setM(showId);
        }
    }

    @Override
    public MallGoodsSku findShowGoods(String showId, String showgoodsId) {
        MallShow b = mallShowMapper.selectByPrimaryKey(showId);
        if(ObjectUtils.allNotNull(b)) {
            if(b.getContent().getE().findSkuExist(showgoodsId)) {
                MallGoodsSku s = mallGoodsSkuMapper.selectByPrimaryKey(showgoodsId);
                return s;
            }
        }
        return null;
    }

    @Override
    public Boolean doPublishShow(String showId) {
        MallShow b = mallShowMapper.selectByPrimaryKey(showId);
        if(ObjectUtils.allNotNull(b)) {
            b.setState(CommonConst.StateEnum.Enable.getKey());
            b.setGmtPublish(new Date());
            MallUtils.fillBeanBeforeUpdate(null,b);
            mallShowMapper.updateByPrimaryKey(b);
            return Boolean.TRUE;
        } else {
            throw new AppException("秀不存在").setM(showId);
        }
    }

    @Override
    public Boolean deleteShow(String showId) {
        MallShow b = mallShowMapper.selectByPrimaryKey(showId);
        if(ObjectUtils.allNotNull(b)) {
            mallShowMapper.deleteByPrimaryKey(showId);
            return Boolean.TRUE;
        } else {
            throw new AppException("秀不存在").setM(showId);
        }
    }

    protected MallMerchantMember createMallMerchantMember(String merchantId, String name, String phone, String accountId) {
        if(StringUtils.isAnyBlank(merchantId,name,phone)) {
            throw new AppException("商户归属、姓名、电话不能为空").setM(JsonUtils.toJson(Lists.newArrayList(merchantId,name,phone)));
        }
        MallMerchant m = mallMerchantMapper.selectByPrimaryKey(merchantId);
        if(ObjectUtils.allNull(m)) {
            throw new AppException("商户不存在").setM(merchantId);
        }
        MallMerchantMember pm = findMallMerchantMemberByPhone(phone);
        if(ObjectUtils.allNotNull(pm)) {
            throw new AppException("该手机号已注册").setM(phone);
        }
        if(StringUtils.isNotBlank(accountId)) {
            List<MallMerchantMember> members = findMembersByAccountId(accountId);
            if(CollectionUtils.isNotEmpty(members)) {
                throw new AppException("该账户已绑定商户，不允许多次绑定").setM(accountId);
            }
            accountService.doAccountInfoFill(accountId,name,phone);
        }
        MallMerchantMember member = new MallMerchantMember();
        MallUtils.fillBeanBeforeInsert(member);
        member.setAccountId(accountId);
        member.setName(name);
        member.setPhone(phone);
        member.setMerchantId(merchantId);
        mallMerchantMemberMapper.insert(member);
        if(StringUtils.isBlank(accountId)) {
            log.info("商户用户["+member.getId()+"]创建成功，未绑定登录");
        } else {
            log.info("商户用户["+member.getId()+"]创建成功，绑定了登录["+accountId+"]");
        }

        if(StringUtils.isBlank(m.getOwner())){
            m.setOwner(member.getId());
            MallUtils.fillBeanBeforeUpdate(null,m);
            mallMerchantMapper.updateByPrimaryKey(m);
            log.info("商户["+m.getId()+"]自动绑定拥有者["+member.getId()+"]");
        }
        return member;
    }

    protected MallMerchantMember updateMallMerchantMember(String memberId, String name, String phone) {
        if(StringUtils.isAnyBlank(name,phone)) {
            throw new AppException("姓名、电话不能为空").setM(JsonUtils.toJson(Lists.newArrayList(name,phone)));
        }
        MallMerchantMember member = findMemberById(memberId);
        if(member == null) {
            throw new AppException("该商户员工不存在");
        }
        MallMerchantMember pm = findMallMerchantMemberByPhone(phone);
        if(pm != null && !pm.getId().equals(member.getId())) {
            throw new AppException("该手机号已被他人使用").setM(phone);
        }
        if(StringUtils.isNotBlank(member.getAccountId())) {
            accountService.doAccountInfoFill(member.getAccountId(),name,phone);
        }
        MallUtils.fillBeanBeforeUpdate(null,member);
        member.setName(name);
        member.setPhone(phone);
        mallMerchantMemberMapper.updateByPrimaryKey(member);
        return member;
    }

    protected MallMerchantMember findMallMerchantMemberByPhone(String phone) {
        if(StringUtils.isBlank(phone)) {return null;}
        MallMerchantMemberExample e = new MallMerchantMemberExample();
        e.createCriteria().andPhoneEqualTo(phone);
        List<MallMerchantMember> ms = mallMerchantMemberMapper.selectByExample(e);
        return CollectionUtils.isEmpty(ms)?null:ms.stream().findFirst().get();
    }

//    protected MallMerchant findMallMerchantByName(String company, String ext1) {
//        if(StringUtils.isAnyBlank(company,ext1)) {
//            throw new AppException("商户名，联系人不能为空");
//        }
//        MallMerchantExample e = new MallMerchantExample();
//        e.createCriteria().andNameEqualTo(company).andExt1EqualTo(ext1);
//        List<MallMerchant> ms = mallMerchantMapper.selectByExample(e);
//        return CollectionUtils.isEmpty(ms)?null:ms.stream().findFirst().get();
//    }

    protected MallMerchant findMallMerchantByName(String company) {
        if(StringUtils.isAnyBlank(company)) {
            throw new AppException("商户名不能为空");
        }
        MallMerchantExample e = new MallMerchantExample();
        e.createCriteria().andNameEqualTo(company);
        List<MallMerchant> ms = mallMerchantMapper.selectByExample(e);
        return CollectionUtils.isEmpty(ms)?null:ms.stream().findFirst().get();
    }

    /**
     * bo的逻辑正确：即category、goods两个集合与bo.getMallShow().getContent()对应得上
     * @param bo
     * @return
     */
    protected MallShowBO createMallShowBO(MallShowBO bo) {
        if(bo == null || bo.getMallShow() == null) {
            throw new AppException("秀不能为空");
        }
        MallUtils.fillBeanBeforeInsert(bo.getMallShow());
        // 准备：基于bo的分类-商品链接做数据分析所需要的临时集合
        Map<String,String> skuIdMap = Maps.newHashMap();
        Map<String,String> goodsIdMap = Maps.newHashMap();
        Map<String,String> cateIdMap = Maps.newHashMap();
        // 保存分类与商品
        if(CollectionUtils.isNotEmpty(bo.getMallGoodsSkus())) {
            for(MallGoodsSku sku:bo.getMallGoodsSkus()) {
                String oid = sku.getId();
                MallGoodsSku saveSku = createMallGoodsSku(sku);
                skuIdMap.put(oid,saveSku.getId());
                goodsIdMap.put(oid,saveSku.getGoodsId());
            }
        }
        if(CollectionUtils.isNotEmpty(bo.getMallCategories())) {
            for(MallCategory c:bo.getMallCategories()) {
                String oid = c.getId();
                MallCategory savec = saveMallCategory(c);
                cateIdMap.put(oid, savec.getId());
            }
        }
        // 分析：新的关联数据
        MallShowContent content = bo.getMallShow().getContent().getE();
        for(MallShowGoods g:content.getGoods()) {
            g.setSkuId(skuIdMap.get(g.getSkuId()));
            g.setGoodsId(goodsIdMap.get(g.getSkuId()));
            g.setCategoryId(cateIdMap.get(g.getCategoryId()));
        }
        for(int i=0,s=content.getCategories().size();i<s;i++) {
            String oldId = content.getCategories().get(i);
            content.getCategories().remove(i);
            content.getCategories().add(i,cateIdMap.get(oldId));
        }
        mallShowMapper.insert(bo.getMallShow());
        return findShowById(bo.getMallShow().getId());
    }

    protected MallCategory saveMallCategory(MallCategory category) {
        MallCategoryExample e = new MallCategoryExample();
        // 关键点：是根据名字匹配，来保证一个商户下的分类是统一、不重复的
        e.createCriteria().andNameEqualTo(category.getName()).andMerchantIdEqualTo(category.getMerchantId());
        List<MallCategory> cs = mallCategoryMapper.selectByExample(e);
        if(CollectionUtils.isNotEmpty(cs)) {
            // 该商户下，有同样名字的就直接返回它
            return cs.stream().findFirst().get();
        } else {
            // 该商户下，没有就新建一个
            MallUtils.fillBeanBeforeInsert(category);
            mallCategoryMapper.insert(category);
            return category;
        }
    }

    protected MallGoodsSku saveMallGoodsSku(MallGoodsSku sku) {
        if(sku == null) {
            throw new AppException("商品不能为空");
        }
        MallGoodsSku dbsku = mallGoodsSkuMapper.selectByPrimaryKey(sku.getId());
        if(dbsku != null) {
            return updateMallGoodsSku(sku);
        } else {
            // 新增
            return createMallGoodsSku(sku);
        }
    }

    protected MallGoodsSku updateMallGoodsSku(MallGoodsSku sku) {
        if(sku == null) {
            throw new AppException("商品不能为空");
        }
        MallGoodsSku dbsku = mallGoodsSkuMapper.selectByPrimaryKey(sku.getId());
        if(dbsku == null) {
            throw new AppException("商品不存在").setM(sku.getId());
        }
        // 更新
        MallUtils.fillBeanBeforeUpdate(dbsku, sku);
        if(StringUtils.isNotBlank(sku.getTopicImage())) {
            spaceService.doFormal(sku.getTopicImage());
        }
        if(StringUtils.isNotBlank(sku.getDesImages())) {
            List<String> images = Lists.newArrayList(StringUtils.split(sku.getDesImages(), CommonConst.COMMA_STRING));
            spaceService.doFormal(images);
        }
        mallGoodsSkuMapper.updateByPrimaryKey(sku);
        return sku;
    }

    protected MallGoodsSku createMallGoodsSku(MallGoodsSku sku) {
        if(sku == null) {
            throw new AppException("商品sku不能为空");
        }
        MallMerchant merchant = mallMerchantMapper.selectByPrimaryKey(sku.getMerchantId());
        if(merchant == null) {
            throw new AppException("商户不存在").setM(sku.getMerchantId());
        }
        // 先查找对应的商品信息
        MallGoods goods = null;
        if( StringUtils.isNotBlank(sku.getGoodsId())) {
            // 先根据ID查找
            goods = mallGoodsMapper.selectByPrimaryKey(sku.getGoodsId());
        }
        if( goods == null ) {
            // 再根据name来匹配商品
            MallGoodsExample e = new MallGoodsExample();
            e.createCriteria().andNameEqualTo(sku.getName()).andMerchantIdEqualTo(sku.getMerchantId());
            List<MallGoods> gs = mallGoodsMapper.selectByExample(e);
            goods = CollectionUtils.isEmpty(gs)?null:gs.stream().findFirst().get();
        }
        if( goods == null ) {
            // 最后，实在找不到商品，就新建一个对应商品
            goods = new MallGoods();
            MallUtils.fillBeanBeforeInsert(goods);
            goods.setName(sku.getName());
            goods.setMerchantId(sku.getMerchantId());
            mallGoodsMapper.insert(goods);
        }
        // 正式保存SKU
        MallUtils.fillBeanBeforeInsert(sku);
        sku.setGoodsId(goods.getId());
        if(StringUtils.isNotBlank(sku.getTopicImage())) {
            spaceService.doFormal(sku.getTopicImage());
        }
        if(StringUtils.isNotBlank(sku.getDesImages())) {
            List<String> images = Lists.newArrayList(StringUtils.split(sku.getDesImages(), CommonConst.COMMA_STRING));
            spaceService.doFormal(images);
        }
        // 保存
        mallGoodsSkuMapper.insert(sku);
        return sku;
    }

    protected MallShowBO fillMallShowBO(MallShow bean) {
        List<MallShow> beans = ObjectUtils.allNull(bean)?null: Lists.newArrayList(bean);
        return fillMallShowBOs(beans).stream().findFirst().orElse(null);
    }

    protected List<MallShowBO> fillMallShowBOs(List<MallShow> beans) {
        Map<String,MallShowBO> r = Maps.newLinkedHashMap();
        if(!CollectionUtils.isEmpty(beans)) {
            // 预备
            Set<String> cIds = Sets.newHashSet();
            Set<String> sIds = Sets.newHashSet();
            beans.forEach(bean->{
                r.putIfAbsent(bean.getId(),new MallShowBO(bean));
                if(ObjectUtils.allNotNull(bean.getContent().getE())) {
                    if(CollectionUtils.isNotEmpty(bean.getContent().getE().getGoods())) {
                        for(MallShowGoods goods:bean.getContent().getE().getGoods()) {
                            sIds.add(goods.getSkuId());
                        }
                    }
                    if(CollectionUtils.isNotEmpty(bean.getContent().getE().getCategories())) {
                        cIds.addAll(bean.getContent().getE().getCategories());
                    }
                }
            });
            Map<String,MallGoodsSku> skuMap = Maps.newHashMap();
            // 查询
            if(CollectionUtils.isNotEmpty(sIds)) {
                MallGoodsSkuExample e = new MallGoodsSkuExample();
                e.createCriteria().andIdIn(Lists.newArrayList(sIds));
                List<MallGoodsSku> skus = mallGoodsSkuMapper.selectByExample(e);
                for(MallGoodsSku sku:skus) {
                    skuMap.put(sku.getId(),sku);
                }
            }

            Map<String,MallCategory> categoryMap = Maps.newHashMap();
            if(CollectionUtils.isNotEmpty(cIds)) {
                MallCategoryExample ce = new MallCategoryExample();
                ce.createCriteria().andIdIn(Lists.newArrayList(cIds));
                List<MallCategory> categories = mallCategoryMapper.selectByExample(ce);
                for(MallCategory c:categories) {
                    categoryMap.put(c.getId(), c);
                }
            }

            // 装填
            for (Map.Entry<String, MallShowBO> en : r.entrySet()) {
                MallShowBO bo = en.getValue();
                for(String cid:bo.findCategoryIds()) {
                    bo.getMallCategories().add(categoryMap.get(cid));
                }
                for(String sid:bo.findGoodsSkuIds()) {
                    bo.getMallGoodsSkus().add(skuMap.get(sid));
                }
            }
        }
        return Lists.newArrayList(r.values());
    }

    protected MallMerchantBO fillMallMerchantBO(MallMerchant bean) {
        List<MallMerchant> beans = ObjectUtils.allNull(bean)?null: Lists.newArrayList(bean);
        return fillMallMerchantBOs(beans).stream().findFirst().orElse(null);
    }

    protected List<MallMerchantBO> fillMallMerchantBOs(List<MallMerchant> beans) {
        Map<String,MallMerchantBO> r = Maps.newLinkedHashMap();
        if(!CollectionUtils.isEmpty(beans)) {
            // 预备
            beans.forEach(b->r.put(b.getId(),new MallMerchantBO(b)));
            // 查询
            Map<String,List<MallMerchantMember>> memberMap = Maps.newHashMap();
            MallMerchantMemberExample e = new MallMerchantMemberExample();
            e.createCriteria().andMerchantIdIn(Lists.newArrayList(r.keySet()));
            List<MallMerchantMember> members = mallMerchantMemberMapper.selectByExample(e);
            for(MallMerchantMember m:members) {
                memberMap.computeIfAbsent(m.getMerchantId(),k->Lists.newArrayList()).add(m);
            }

            // 装填
            for (Map.Entry<String, MallMerchantBO> en : r.entrySet()) {
                List<MallMerchantMember> ms = memberMap.get(en.getKey());
                if(CollectionUtils.isNotEmpty(ms)) {
                    en.getValue().getMallMerchantMembers().addAll(ms);
                }
            }
        }
        return Lists.newArrayList(r.values());
    }
}
